@@ -14,7 +14,7 @@
 static PyObject* setColor( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -26,21 +26,21 @@ static PyObject* setColor( PyObject* self, PyObject* args )
         "OO", &py_surface, &py_color
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
 
     if(!RGBAFromColorObj (py_color, color))
         return Py_None;
-    
+
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
-            pixel = pixels[(y*w)+x];
+            pixel = getPixel(surface, x, y);
             SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
             if(a==0) continue;
             pixel = SDL_MapRGBA( format, color[0], color[1], color[2], a );
-            pixels[ (y*w)+x ] = pixel;
+            setPixel(surface, x, y, pixel);
         }
     }
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 
@@ -56,7 +56,7 @@ static PyObject* setColor( PyObject* self, PyObject* args )
 static PyObject* setAlpha( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -69,7 +69,7 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     if( method == 1 ){
         shift = clampUint8( shift );
     }
@@ -79,7 +79,7 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
-            pixel = pixels[(y*w)+x];
+            pixel = getPixel(surface, x, y);
             SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
             if(method == 0){
                 a = a + shift;
@@ -92,10 +92,10 @@ static PyObject* setAlpha( PyObject* self, PyObject* args )
                 a = a * percent;
             }
             pixel = SDL_MapRGBA( format, r, g, b, a );
-            pixels[ (y*w)+x ] = pixel;
+            setPixel(surface, x, y, pixel);
         }
     }
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 
@@ -112,11 +112,10 @@ static PyObject* greyscale( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
     SDL_PixelFormat *format = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h = 0;
-    int pxcnt = 0;
-    Uint32 pixel;
     Uint8 r,g,b,a;
+    int x,y;
     double hue,sat,val;
     int v;
     int method = 0;
@@ -125,37 +124,23 @@ static PyObject* greyscale( PyObject* self, PyObject* args )
         "O|i", &py_surface, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
-    pxcnt = w * h;
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
 
-    if(method == 0){
-        while(pxcnt--){
-            pixel = pixels[pxcnt];
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
-            /* average RGB */
-            v = (r+g+b)/3.0;
-            pixel = SDL_MapRGBA( format, v, v, v, a );
-            pixels[pxcnt] = pixel;
-        }
-    } else if(method == 1){
-        while(pxcnt--){
-            pixel = pixels[pxcnt];
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
-            /* HSV stauration = 0 */
-            RGB2HSV( r, g, b, &hue, &sat, &val );
-            HSV2RGB( hue, 0, val, &r, &g, &b );
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            pixels[pxcnt] = pixel;
-        }
-    } else if(method == 2){
-        while(pxcnt--){
-            pixel = pixels[pxcnt];
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
-            /* HSL stauration = 0 */
-            RGB2HSL( r, g, b, &hue, &sat, &val );
-            HSL2RGB( hue, 0, val, &r, &g, &b );
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            pixels[pxcnt] = pixel;
+    for(x = 0; x < w; x ++){
+        for(y = 0; y < h; y ++){
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a);
+            if(method == 0){
+                v = (r+g+b)/3.0;
+                setPixel_RGBA(surface, x, y, v, v, v, a);
+            } else if(method == 1){
+                RGB2HSV( r, g, b, &hue, &sat, &val );
+                HSV2RGB( hue, 0, val, &r, &g, &b );
+                setPixel_RGBA(surface, x, y, r, g, b, a);
+            } else if(method == 2){
+                RGB2HSL( r, g, b, &hue, &sat, &val );
+                HSL2RGB( hue, 0, val, &r, &g, &b );
+                setPixel_RGBA(surface, x, y, r, g, b, a);
+            }
         }
     }
 
@@ -170,7 +155,7 @@ static PyObject* greyscale( PyObject* self, PyObject* args )
 static PyObject* invert( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,pxcnt = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -180,14 +165,14 @@ static PyObject* invert( PyObject* self, PyObject* args )
         "O", &py_surface
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     while( pxcnt-- ){
-        pixel = pixels[pxcnt];
+        pixel = getPixelN(surface, pxcnt);
         SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
         pixel = SDL_MapRGBA( format, 255-r, 255-g, 255-b, a );
-        pixels[pxcnt] = pixel;
+        setPixelN(surface, pxcnt, pixel);
     }
     Py_INCREF(Py_None);
     return Py_None;
@@ -201,7 +186,7 @@ static PyObject* invert( PyObject* self, PyObject* args )
 static PyObject* hue( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,pxcnt = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -214,11 +199,11 @@ static PyObject* hue( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     while( pxcnt-- ){
-        pixel = pixels[pxcnt];
+        pixel = getPixelN(surface, pxcnt);
         SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
 
         RGB2HSV( r,g,b, &hue, &sat, &val );
@@ -239,7 +224,7 @@ static PyObject* hue( PyObject* self, PyObject* args )
         hue = clampValue( hue, 0.0, 360.0 );
         HSV2RGB( hue, sat, val, &r, &g, &b );
         pixel = SDL_MapRGBA( format, r, g, b, a );
-        pixels[pxcnt] = pixel;
+        setPixelN(surface, pxcnt, pixel);
     }
     Py_INCREF(Py_None);
     return Py_None;
@@ -256,7 +241,7 @@ static PyObject* hue( PyObject* self, PyObject* args )
 static PyObject* saturation( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,pxcnt = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -269,15 +254,16 @@ static PyObject* saturation( PyObject* self, PyObject* args )
     PY_PARSEARGS_BEGIN
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
-    
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if( method == 2 ){
         percent = ((double)(shift))/100.0;
     }
+
     while( pxcnt-- ){
-        pixel = pixels[pxcnt];
+        pixel = getPixelN(surface, pxcnt);
         SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
 
         RGB2HSV( r,g,b, &hue, &sat, &val );
@@ -285,13 +271,13 @@ static PyObject* saturation( PyObject* self, PyObject* args )
             case 0: sat += (double)(shift);break;
             case 1: sat = (double)(shift);break;
             case 2: sat = sat * percent;break;
-        }        
-        
+        }
+
         /* clamp sat 0-100 */
         sat = clampValue( sat, 0.0, 100.0 );
         HSV2RGB( hue,sat,val, &r, &g, &b );
         pixel = SDL_MapRGBA( format, r, g, b, a );
-        pixels[pxcnt] = pixel;
+        setPixelN(surface, pxcnt, pixel);
     }
     Py_INCREF(Py_None);
     return Py_None;
@@ -308,7 +294,7 @@ static PyObject* saturation( PyObject* self, PyObject* args )
 static PyObject* value( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,pxcnt = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -323,14 +309,14 @@ static PyObject* value( PyObject* self, PyObject* args )
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if( method == 2 ){
         percent = ((double)(shift))/100.0;
     }
     while( pxcnt-- ){
-        pixel = pixels[pxcnt];
+        pixel = getPixelN(surface, pxcnt);
         SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
 
         RGB2HSV( r,g,b, &hue, &sat, &val );
@@ -339,12 +325,12 @@ static PyObject* value( PyObject* self, PyObject* args )
             case 1: val = (double)(shift);break;
             case 2: val = val * percent;break;
         }
-        
+
         /* clamp sat 0-100 */
         val = clampValue( val, 0, 100 );
         HSV2RGB( hue,sat,val, &r, &g, &b );
         pixel = SDL_MapRGBA( format, r, g, b, a );
-        pixels[pxcnt] = pixel;
+        setPixelN(surface, pxcnt, pixel);
     }
     Py_INCREF(Py_None);
     return Py_None;
@@ -354,7 +340,7 @@ static PyObject* value( PyObject* self, PyObject* args )
 static PyObject* lightness( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,pxcnt = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -367,15 +353,15 @@ static PyObject* lightness( PyObject* self, PyObject* args )
     PY_PARSEARGS_BEGIN
         "Oi|i", &py_surface, &shift, &method
     PY_PARSEARGS_END
-    
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if( method == 2 ){
         percent = ((double)(shift))/100.0;
     }
     while( pxcnt-- ){
-        pixel = pixels[pxcnt];
+        pixel = getPixelN(surface, pxcnt);
         SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
 
         RGB2HSL( r,g,b, &hue, &sat, &val );
@@ -384,12 +370,12 @@ static PyObject* lightness( PyObject* self, PyObject* args )
             case 1: val = (double)(shift);break;
             case 2: val = val * percent;break;
         }
-        
+
         /* clamp sat 0-100 */
         val = clampValue( val, 0.0, 100.0 );
         HSL2RGB( hue,sat,val, &r, &g, &b );
         pixel = SDL_MapRGBA( format, r, g, b, a );
-        pixels[pxcnt] = pixel;
+        setPixelN(surface, pxcnt, pixel);
     }
     Py_INCREF(Py_None);
     return Py_None;
@@ -400,7 +386,7 @@ static PyObject* lightness( PyObject* self, PyObject* args )
 static PyObject* multiply( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,pxcnt = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -412,8 +398,8 @@ static PyObject* multiply( PyObject* self, PyObject* args )
     PY_PARSEARGS_BEGIN
         "Of|bbb", &py_surface, &shift, &rb, &gb, &bb
     PY_PARSEARGS_END
-    
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     pxcnt = w * h;
 
     if(rb){rp = shift;}
@@ -421,7 +407,7 @@ static PyObject* multiply( PyObject* self, PyObject* args )
     if(bb){bp = shift;}
 
     while( pxcnt-- ){
-        pixel = pixels[pxcnt];
+        pixel = getPixelN(surface, pxcnt);
         SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
 
         r = clampUint8(r * rp);
@@ -429,7 +415,7 @@ static PyObject* multiply( PyObject* self, PyObject* args )
         b = clampUint8(b * bp);
 
         pixel = SDL_MapRGBA( format, r, g, b, a );
-        pixels[pxcnt] = pixel;
+        setPixelN(surface, pxcnt, pixel);
     }
     Py_INCREF(Py_None);
     return Py_None;
@@ -442,7 +428,7 @@ static PyObject* multiply( PyObject* self, PyObject* args )
 static PyObject* colorize( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -455,15 +441,15 @@ static PyObject* colorize( PyObject* self, PyObject* args )
     long count = 0;
 
     PY_PARSEARGS_BEGIN
-        "Oii|ii", 
+        "Oii|ii",
         &py_surface, &check_hue, &new_hue, &v_shift, &s_shift
     PY_PARSEARGS_END
-    
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
-            pixel = pixels[(y*w)+x];
+            pixel = getPixel(surface, x, y);
             SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
             RGB2HSV( r,g,b, &hue, &sat, &val );
             if( (int)hue == check_hue ){
@@ -483,7 +469,7 @@ static PyObject* colorize( PyObject* self, PyObject* args )
             }
             HSV2RGB( hue,sat,val, &r, &g, &b );
             pixel = SDL_MapRGBA( format, r, g, b, a );
-            pixels[ (y*w)+x ] = pixel;
+            setPixel(surface, x, y, pixel);
 
         }
     }
@@ -497,12 +483,13 @@ static PyObject* colorize( PyObject* self, PyObject* args )
 static PyObject* alphaMask( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    PyObject *out_py_surface = NULL;
+    SDL_Surface *surface;
+    SDL_Surface *target_surface;
     int w,h,x,y,mw,mh = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
     PyObject* py_surface_mask = NULL;
-    Uint32 *pixels_mask;
+    SDL_Surface *surface_mask;
     SDL_PixelFormat *format_mask = NULL;
 
     Uint8 r,g,b,a,tmp_a;
@@ -514,10 +501,16 @@ static PyObject* alphaMask( PyObject* self, PyObject* args )
         "OO|i",
         &py_surface, &py_surface_mask, &method
     PY_PARSEARGS_END
-    
-    getSurfaceAttrs(py_surface_mask, &mw, &mh, &format_mask, &pixels_mask);
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
-    
+
+    getSurfaceAttrsSurf(py_surface_mask, &mw, &mh, &format_mask, &surface_mask);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+    target_surface = createSurface( mw, mh );
+
+
+    /*if(format->BytesPerPixel == 3){
+        return RAISE (PyExc_ValueError, "input surface must be 32bit surface not 24bit");
+    }*/
+
     if( mw != w || mh != h ){
         return RAISE (PyExc_ValueError, "mask must be same size as surface");
     }
@@ -525,61 +518,55 @@ static PyObject* alphaMask( PyObject* self, PyObject* args )
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
             /* get mask pixel*/
-            pixel = pixels_mask[(y*w)+x];
-            SDL_GetRGBA( pixel, format_mask, &r, &g, &b, &a );
+            getPixel_RGBA(surface_mask, x, y, &r, &g, &b, &a );
+
             if( method == 0 ){
                 RGB2HSV( r,g,b, &hue, &sat, &val );
-                if(val >= 99.9) continue;
-                alpha = (float)(val/100.0);
+                alpha = (((float)val) / 100.0);
                 /* set pixel */
-                pixel = pixels[(y*w)+x];
-                SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
-                
-                tmp_a = (Uint8)(((float)a) * alpha );
+                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
+                tmp_a = (Uint8)( ((float)a) * alpha );
             } else if( method == 1 ){
                 tmp_a = a;
-                pixel = pixels[(y*w)+x];
-                SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
             } else if( method == 2 ){
                 tmp_a = a;
-                pixel = pixels[(y*w)+x];
-                SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
                 /* alpha blend */
                 alpha = ((float) a) / 255.0;
                 alpha2 = ((float) tmp_a) / 255.0;
                 tmp_a = (alpha * alpha2) * 255;
             } else if( method == 3 ){
                 tmp_a = a;/*mask alpha*/
-                pixel = pixels[(y*w)+x];
-                SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+                getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
                 if(tmp_a > 0){/* if mask alpha > 0 then set original alpha */
                     tmp_a = a;
                 }
             } else {
                 tmp_a = 255;
             }
-            
-            pixel = SDL_MapRGBA( format, r, g, b, tmp_a );
-            pixels[ (y*w)+x ] = pixel;
+
+            setPixel_RGBA(target_surface, x, y, r, g, b, tmp_a);
 
         }
     }
-    
-    return Py_None;
 
+    out_py_surface = PySurface_New( target_surface );
+    Py_INCREF( out_py_surface );
+    return Py_BuildValue("O", out_py_surface );
 }
 
 
 static PyMethodDef ColorMethods[] = {
     {"setColor", setColor, METH_VARARGS, "Set opaque pixels to color"},
     {"setAlpha", setAlpha, METH_VARARGS, "Set alpha channel of surface"},
-    
+
     {"hue", hue, METH_VARARGS, "Hue"},
     {"saturation", saturation, METH_VARARGS, "Change saturation of surface"},
     {"value", value, METH_VARARGS, "Change value of surface"},
     {"lightness", lightness, METH_VARARGS, "Lightness"},
     {"multiply", multiply, METH_VARARGS, "Color multiply"},
-    
+
     {"invert", invert, METH_VARARGS, "Invert colors of surface"},
     {"greyscale", greyscale, METH_VARARGS, "Create greyscale surface"},
     {"colorize", colorize, METH_VARARGS, "Change hue of surface"},
