@@ -3,7 +3,7 @@
     Author: Josef Vanžura
     Required: Python 2.5+, PyGame 1.8.1+, SDL 1.2
 */
-    
+
 #include "pgext.h"
 
 #define PGEXT_VERSION 6
@@ -12,28 +12,23 @@
 
 /* simple horizontal/vertical blur */
 static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char vertical ){
-    Uint32 pix;
     Uint8 r,g,b,a;
     double sr,sg,sb,sa;
     int color_cnt = 0;
-    int x,y,ra,n,s,o1,o2;
-    double r2;
+    int x,y,ra,n,s,o1,o2,pxn;
+    //double r2;
     int pxcnt = 0;
 
     /* Get sdl_surface attributes */
     int w = source->w;
     int h = source->h;
-    SDL_PixelFormat *source_format = (SDL_PixelFormat *)source->format;
-    SDL_PixelFormat *target_format = (SDL_PixelFormat *)target->format;
-    Uint32 *source_pixels = (Uint32 *) source->pixels;
-    Uint32 *target_pixels = (Uint32 *) target->pixels;
 
     /* Vertical/horizonal depending values */
     s = (vertical)?h:w;
     o1 = (vertical)?1:0;
     o2 = (vertical)?0:1;
-    r2 = radius*2+1;
-    
+    //r2 = radius*2+1;
+
     SDL_LockSurface(source);
     SDL_LockSurface(target);
     for( y = 0; y < h; y++ ){
@@ -45,8 +40,8 @@ static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char ve
             /* Sum of pixel within radius */
             while( ra <= radius ){
                 if( (n+ra) >= 0 && (n+ra) < s ){
-                    pix = source_pixels[pxcnt + (int)( ((ra * o1) * w) + (ra * o2))];
-                    SDL_GetRGBA( pix, source_format, &r, &g, &b, &a );
+                    pxn = pxcnt + (int)( ((ra * o1) * w) + (ra * o2));
+                    getPixelN_RGBA(source, pxn, &r, &g, &b, &a );
                     sr += r;sg += g;sb += b;sa += a;
                     color_cnt++;
                 }
@@ -63,9 +58,7 @@ static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char ve
             sa = sa / color_cnt;
 
             /* map & set pixel */
-            pix = SDL_MapRGBA( target_format, (Uint8)sr, (Uint8)sg, 
-                (Uint8)sb, (Uint8)sa);
-            target_pixels[pxcnt-1] = pix;
+            setPixel_RGBA(target, x, y, (Uint8)sr, (Uint8)sg, (Uint8)sb, (Uint8)sa);
         }
     }
     SDL_UnlockSurface(source);
@@ -79,7 +72,7 @@ static void _blur( SDL_Surface *source, SDL_Surface *target, int radius, char ve
 static PyObject* noise( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -93,13 +86,13 @@ static PyObject* noise( PyObject* self, PyObject* args )
         "O|ii", &py_surface, &value, &density
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     //srand();
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
             if( rand()%density != 0 ) continue;
-            pixel = pixels[(y*w)+x];
+            pixel = getPixel(surface, x, y);
             SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
 
             rnd = (value / 2) - (rand() % value);
@@ -109,13 +102,12 @@ static PyObject* noise( PyObject* self, PyObject* args )
             r = clampUint8(tr);
             g = clampUint8(tg);
             b = clampUint8(tb);
-            
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            pixels[ (y*w)+x ] = pixel;
 
+            pixel = SDL_MapRGBA( format, r, g, b, a );
+            setPixel(surface, x, y, pixel);
         }
     }
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -127,21 +119,20 @@ static PyObject* noise( PyObject* self, PyObject* args )
 static PyObject* noiseBlur( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y = 0;
-    Uint32 pixel;
     SDL_PixelFormat *format = NULL;
 
     int radius = 5;
     char blend = 0;
     Uint8 r,g,b,a,tr,tg,tb,ta;
-    int rnd,dx,dy;
+    int dx,dy;
 
     PY_PARSEARGS_BEGIN
         "O|ib", &py_surface, &radius, &blend
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
     //srand();
 
     for( y = 0; y < h; y ++ ){
@@ -150,21 +141,17 @@ static PyObject* noiseBlur( PyObject* self, PyObject* args )
             dy = y + ((rand() % (radius * 2)) - radius);
             if( dx < 0 || dx >= w || dx == x ) continue;
             if( dy < 0 || dy >= h || dy == y ) continue;
-            pixel = pixels[(dy*w)+dx];
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+            getPixel_RGBA(surface, dx, dy, &r, &g, &b, &a );
             if(blend){
-                pixel = pixels[(y*w)+x];
-                SDL_GetRGBA( pixel, format, &tr, &tg, &tb, &ta );
+                getPixel_RGBA(surface, x, y, &tr, &tg, &tb, &ta );
                 r = (tr + r)/2;
                 g = (tg + g)/2;
                 b = (tb + b)/2;
             }
-            pixel = SDL_MapRGBA( format, r, g, b, a );
-            pixels[ (y*w)+x ] = pixel;
-
+            setPixel_RGBA(surface, x, y, r, g, b, a);
         }
     }
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -176,8 +163,7 @@ static PyObject* noiseBlur( PyObject* self, PyObject* args )
 static PyObject* scratch( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    PySurfaceObject* c_surface;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -186,17 +172,14 @@ static PyObject* scratch( PyObject* self, PyObject* args )
     int rnd,tx;
 
     SDL_Surface *surface_copy = NULL;
-    Uint32 *pixels_copy;
 
     PY_PARSEARGS_BEGIN
         "O|i", &py_surface, &offset
     PY_PARSEARGS_END
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
 
-    c_surface = (PySurfaceObject *) py_surface;
-    surface_copy = SDL_ConvertSurface(c_surface->surf, format, 0);
-    pixels_copy = (Uint32 *)surface_copy->pixels;
+    surface_copy = SDL_ConvertSurface(surface, format, 0);
 
     //srand();
 
@@ -206,11 +189,11 @@ static PyObject* scratch( PyObject* self, PyObject* args )
             tx = rnd + x;
             if( tx < 0 ) continue;
             if( tx >= w ) continue;
-            pixel = pixels_copy[(y*w)+x];
-            pixels[ (y*w)+tx ] = pixel;
+            pixel = getPixel(surface_copy, x, y);
+            setPixel(surface, tx, y, pixel);
         }
     }
-    
+
     SDL_FreeSurface(surface_copy);
     Py_INCREF(Py_None);
     return Py_None;
@@ -222,8 +205,7 @@ static PyObject* scratch( PyObject* self, PyObject* args )
 static PyObject* pixelize( PyObject* self, PyObject* args )
 {
     PyObject* py_surface = NULL;
-    PySurfaceObject* c_surface;
-    Uint32 *pixels;
+    SDL_Surface *surface;
     int w,h,x,y,rx,ry = 0;
     Uint32 pixel;
     SDL_PixelFormat *format = NULL;
@@ -237,19 +219,21 @@ static PyObject* pixelize( PyObject* self, PyObject* args )
 
     halfoffset = (int) (((float) offset) / 2);
 
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
 
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
             rx = ((x / offset) * offset) + halfoffset;
             ry = ((y / offset) * offset) + halfoffset;
+            if(x == rx && y == ry) continue;
             if(rx >= w) rx = w - 1;
             if(ry >= h) ry = h - 1;
-            pixel = pixels[(ry*w)+rx];
-            pixels[ (y*w)+x ] = pixel;
+            pixel = getPixel(surface, rx, ry);
+            setPixel(surface, x, y, pixel);
+
         }
     }
-    
+
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -264,7 +248,6 @@ static PyObject* gradient( PyObject* self, PyObject* args )
     SDL_Surface *surface = NULL;
     PyObject *py_surface = NULL, *py_fcolor = NULL, *py_tcolor = NULL;
     SDL_PixelFormat *format = NULL;
-    Uint32 *pixels = NULL;
 
     Uint32 pixel;
     Uint8 color[4];
@@ -274,28 +257,26 @@ static PyObject* gradient( PyObject* self, PyObject* args )
     char vertical = 0;
     float ratio = 0.0;
     double nn,delta;
-    
-    
+
+
     PY_PARSEARGS_BEGIN
         "(ii)OO|bf", &w, &h, &py_fcolor, &py_tcolor, &vertical, &ratio
     PY_PARSEARGS_END
-    
+
     if(!RGBAFromColorObj (py_fcolor, fcolor))
         return Py_None;
-        
+
     if(!RGBAFromColorObj (py_tcolor, tcolor))
         return Py_None;
 
-
-    surface = createSurface( w, h );
-    pixels = (Uint32 *)surface->pixels;
-    format = surface->format;
-
     dR = (int)(tcolor[0]-fcolor[0]);
     dG = (int)(tcolor[1]-fcolor[1]);
     dB = (int)(tcolor[2]-fcolor[2]);
     dA = (int)(tcolor[3]-fcolor[3]);
 
+    surface = createSurface( w, h );
+    format = surface->format;
+
     rows = vertical?h:w;
     cols = vertical?w:h;
 
@@ -313,11 +294,11 @@ static PyObject* gradient( PyObject* self, PyObject* args )
         for( q = 0; q < cols; q ++ ){
             x = vertical?q:i;
             y = vertical?i:q;
-            pixels[ ( y * w ) + x ] = pixel;
+            setPixel(surface, x, y, pixel);
         }
     }
     SDL_UnlockSurface (surface);
-    
+
     py_surface = PySurface_New( surface );
     Py_INCREF( py_surface );
     return Py_BuildValue("O", py_surface );
@@ -329,22 +310,19 @@ static PyObject* gradient( PyObject* self, PyObject* args )
 static PyObject* shadow(PyObject* self, PyObject* args)
 {
     PyObject *py_surface = NULL, *py_color = NULL;
-    Uint32 *pixels;
-    Uint32 pixel;
-    unsigned int w,h,x,y;
+    SDL_Surface *surface;
+    int w,h,x,y;
     int rw,rh;
     int offset = 0;
     SDL_PixelFormat *format;
     Uint8 r,g,b,a;
 
     SDL_Surface *target_surface = NULL;
-    Uint32 *target_pixels;
-    SDL_PixelFormat *target_format;
     SDL_Surface *temp_surface = NULL;
     PyObject *out_py_surface = NULL;
 
     Uint8 color[4];
-    
+
     int radius = 5;
     float opacity = 1.0;
     char resized = 0;
@@ -352,15 +330,19 @@ static PyObject* shadow(PyObject* self, PyObject* args)
     PY_PARSEARGS_BEGIN
         "OO|ibf", &py_surface, &py_color, &radius, &resized, &opacity
     PY_PARSEARGS_END
-    
+
     if(!RGBAFromColorObj (py_color, color))
         return Py_None;
 
     if( radius > MAX_SHADOW_VALUE )
         radius = MAX_SHADOW_VALUE;
-    
-    getSurfaceAttrs(py_surface, &w, &h, &format, &pixels);
-    
+
+    getSurfaceAttrsSurf(py_surface, &w, &h, &format, &surface);
+
+    if(format->BytesPerPixel == 3){
+        return RAISE (PyExc_ValueError, "input surface must be 32bit surface not 24bit");
+    }
+
     if( resized ){
         /* Set offsets for resized surface */
         rw = w + (radius * 2);
@@ -374,21 +356,17 @@ static PyObject* shadow(PyObject* self, PyObject* args)
 
     /* Helper surfaces */
     target_surface = createSurface( rw, rh );
-    target_pixels  = (Uint32 *)target_surface->pixels;
-    target_format = target_surface->format;
     temp_surface = createSurface( rw, rh );
-    
+
     /* Lock surfaces for direct pixel set */
     SDL_LockSurface (temp_surface);
     SDL_LockSurface (target_surface);
     /* Set color & copy surface to target */
     for( y = 0; y < h; y ++ ){
         for( x = 0; x < w; x ++ ){
-            pixel = pixels[(y*w)+x];
-            SDL_GetRGBA( pixel, format, &r, &g, &b, &a );
+            getPixel_RGBA(surface, x, y, &r, &g, &b, &a );
             a = (Uint8) (((double)a)*opacity);
-            pixel = SDL_MapRGBA( target_format, color[0], color[1], color[2], a );
-            target_pixels[ ( (y+offset) * rw)+( x+offset ) ] = pixel;
+            setPixel_RGBA(target_surface, x + offset, y + offset, color[0], color[1], color[2], a);
         }
     }
     SDL_UnlockSurface (temp_surface);
@@ -398,10 +376,10 @@ static PyObject* shadow(PyObject* self, PyObject* args)
     _blur( target_surface, temp_surface, radius, 0 );
     /* Switch temp => target with vertical blur */
     _blur( temp_surface, target_surface, radius, 1 );
-    
+
     /* Temp must be free */
     SDL_FreeSurface( temp_surface );
-    
+
     out_py_surface = PySurface_New( target_surface );
     Py_INCREF( out_py_surface );
     return Py_BuildValue("O", out_py_surface );
@@ -415,7 +393,7 @@ static PyObject* blur(PyObject* self, PyObject* args)
 
     PyObject* py_surface = NULL;
     PySurfaceObject *surface;
-    unsigned int w,h;
+    int w,h;
 
     SDL_Surface *temp_surface = NULL;
     unsigned int radius = 5;
@@ -424,19 +402,19 @@ static PyObject* blur(PyObject* self, PyObject* args)
     PY_PARSEARGS_BEGIN
         "O|i", &py_surface, &radius
     PY_PARSEARGS_END
-    
+
     surface = (PySurfaceObject*) py_surface;
     w = surface->surf->w;
     h = surface->surf->h;
-    
+
     /* Kopie je potreba kvuli zdrojovym pixelum */
     temp_surface = createSurface( w, h );
-    
+
     /* Switch original => temp with horizontal blur */
     _blur( surface->surf, temp_surface, radius, 0 );
     /* Switch temp => target with vertical blur */
     _blur( temp_surface, surface->surf, radius, 1 );
-    
+
     SDL_FreeSurface( temp_surface );
 
     return Py_None;
@@ -454,21 +432,21 @@ static PyObject* hvBlur(PyObject* self, PyObject* args)
     SDL_PixelFormat *format;
     unsigned int radius = 5;
     char vertical = 0;
-    
+
     /* Parsovani pyArgumentu - radius by melo byt unsigned int */
     PY_PARSEARGS_BEGIN
         "O|ib", &py_surface, &radius, &vertical
     PY_PARSEARGS_END
-    
+
     surface = (PySurfaceObject*) py_surface;
     format = (SDL_PixelFormat *)surface->surf->format;
-    
+
     /* Kopie je potreba kvuli zdrojovym pixelum */
     copy_surface = SDL_ConvertSurface(surface->surf, format, 0);
-    
+
     /* Switch original => temp with vertical blur */
     _blur( copy_surface, surface->surf, radius, vertical );
-    
+
     SDL_FreeSurface(copy_surface);
 
     Py_INCREF( Py_None );
